/*
 * Copyright 2020 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "wGmresAmg.h"
#include <amgcl/amg.hpp>
#include <amgcl/make_solver.hpp>
#include <amgcl/solver/gmres.hpp>
#include <amgcl/coarsening/smoothed_aggregation.hpp>
#include <amgcl/relaxation/ilut.hpp>
#include <amgcl/adapter/eigen.hpp>
#include <amgcl/backend/eigen.hpp>
using namespace tbox;

void GmresAmg::analyze(Eigen::SparseMatrix<double> const &A)
{
    throw std::runtime_error("tbox::GmresAmg::analyze not implemented!\n");
}
void GmresAmg::factorize(Eigen::SparseMatrix<double> const &A)
{
    throw std::runtime_error("tbox::GmresAmg::factorize not implemented!\n");
}
void GmresAmg::solve(Eigen::Map<Eigen::VectorXd> const &b, Eigen::Map<Eigen::VectorXd> &x)
{
    throw std::runtime_error("tbox::GmresAmg::solve not implemented!\n");
}
void GmresAmg::compute(Eigen::SparseMatrix<double> const &A, Eigen::Map<Eigen::VectorXd> const &b, Eigen::Map<Eigen::VectorXd> &x)
{
    // GMRES with AMG (smoothed agreg + ILUT)
    typedef amgcl::backend::eigen<double> Backend;
    typedef amgcl::make_solver<amgcl::amg<Backend, amgcl::coarsening::smoothed_aggregation, amgcl::relaxation::ilut>, amgcl::solver::gmres<Backend>> Solver;

    // Parameters
    Solver::params p;
    p.solver.M = 50;                            // restart
    p.solver.tol = 1e-5;                        // relative tolerance
    p.solver.maxiter = 1000;                    // max. n. iterations
    p.precond.relax.p = 1;                      // fill-in
    p.precond.relax.tau = 1e-6;                 // threshold
    p.precond.ncycle = 2;                       // cycle shape (W)
    p.precond.pre_cycles = 3;                   // number of cycles
    p.precond.coarsening.aggr.eps_strong = 0.5; // AMG coupling

    // Convert to CRS and solve
    Eigen::SparseMatrix<double, Eigen::RowMajor> AA = A;
    Solver sol(AA, p);
    std::tie(it, err) = sol(AA, b, x);
}

void GmresAmg::write(std::ostream &out) const
{
    out << "GMRES-AMG (AMGCL)" << std::endl;
}
