/*
 * Copyright 2020 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef WGMRESAMG_H
#define WGMRESAMG_H

#include "tbox.h"
#include "wLinearSolver.h"
#include <Eigen/Sparse>

namespace tbox
{

/**
 * @brief GMRES with AMG preconditioner interface
 * @authors Adrien Crovato
 * @todo Configure solver in constructor and public methods
 * @todo Find optimal or good-enough parameters
 */
class TBOX_API GmresAmg : public LinearSolver
{
    int it;     ///< number of iterations
    double err; ///< relative error
public:
    GmresAmg() : LinearSolver() {}
    virtual ~GmresAmg() {}

#ifndef SWIG
    virtual void analyze(Eigen::SparseMatrix<double> const &A) override;
    virtual void factorize(Eigen::SparseMatrix<double> const &A) override;
    virtual void solve(Eigen::Map<Eigen::VectorXd> const &b, Eigen::Map<Eigen::VectorXd> &x) override;
    virtual void compute(Eigen::SparseMatrix<double> const &A, Eigen::Map<Eigen::VectorXd> const &b, Eigen::Map<Eigen::VectorXd> &x) override;

    virtual double getError() override { return err; }
    virtual int getIterations() override { return it; }

    virtual void write(std::ostream &out) const override;
#endif
};

} // namespace tbox

#endif // WGMRESAMG_H
