#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright 2020 University of Liège
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


# Test mesh deformation

import numpy as np
import tbox
import tbox.gmsh as gmsh
import fwk
from fwk.wutils import parseargs

def main():

    # timer
    tms = fwk.Timers()
    tms["total"].start()

    # parameters for mesh deformation
    dx = 0.1
    dy = -0.6

    # mesh an beam
    pars = {}
    msh = gmsh.MeshLoader("models/beam.geo",__file__).execute(**pars)
    gmshWriter = tbox.GmshExport(msh)
    # create mesh deformation handler
    mshDef = tbox.MshDeform(msh, tbox.SparseLu(), 2, nthrds=parseargs().k)
    mshDef.setField("internalField")
    mshDef.addFixed(["clamp"])
    mshDef.addMoving(["tip"])
    mshDef.initialize()

    # deform the mesh (dummy translation of "tip" nodes)
    mshDef.savePos()
    msh.nodes[1].pos[0] += dx
    msh.nodes[1].pos[1] += dy
    msh.nodes[2].pos[0] += dx
    msh.nodes[2].pos[1] += dy

    # deform the mesh
    test = tbox.UnitTest()
    tms["deform"].start()
    test.mshDeform(mshDef, 2)
    tms["deform"].stop()
    gmshWriter.save("beam_def")

    # display timers
    tms["total"].stop()
    print("CPU statistics")
    print(tms)

    # eof
    print('')

if __name__ == "__main__":
    main()
